<?php

namespace Drupal\fox\Plugin\FoxCommand;

/**
 * INFO fox command.
 *
 * @FoxCommand(
 *   id = "info",
 *   label = @Translation("List entities,bundles or fields list for entity,bundle. Usage: INFO [entity_id [bundle]]")
 * )
 */
class FoxCommandInfo extends FoxCommandBaseClass {

  /**
   * {@inheritdoc}
   */
  public function execute(array $params, array $variables): array {
    $helper = $this->foxCommandsHelper();
    $all_entity_types = $helper->entityTypeManager()->getDefinitions();

    if (!empty($params)) {
      $entity_id = $params[0];
      $entity = $all_entity_types[$entity_id] ?? NULL;
      if (!$entity) {
        return $this->errorReturn($this->t('Wrong entity type "@entity_id"', [
          '@entity_id' => $entity_id,
        ]));
      }

      $group = $entity->getGroup();
      if ($group === 'content') {
        return $this->fieldDefinitions($params);
      }
      else {
        $header = [$this->t('ID')];
        $data = [];
        $info = $entity->getPropertiesToExport();
        foreach ($info as $field) {
          $data[] = [$field];
        }

        $message[(string) $this->t('IDs')] = [
          'header' => $header,
          'data' => $data,
        ];

        return [
          'message' => $message,
        ];
      }
    }

    $message = [];
    $header = [$this->t('ID'), $this->t('Title'), $this->t('Bundles')];
    $entity_type_bundle_Info = $helper->entityTypeBundleInfo();
    foreach ($all_entity_types as $entity) {
      $group = $entity->getGroup();
      $id = $entity->id();
      $label = (string) $entity->getLabel();

      $bundle_data = '';
      $bundle_info = $entity_type_bundle_Info->getBundleInfo($id);
      if (!empty($bundle_info) and count($bundle_info) > 1) {
        $bundles = [];
        foreach ($bundle_info as $bundle_id => $bundle) {
          $bundle_label = (string) $bundle['label'];
          $bundles[] = sprintf('%s(%s)', $bundle_id, $bundle_label);
        }
        $bundle_data = implode(',', $bundles);
      }
      $message[$group]['header'] = $header;
      $message[$group]['data'][] = ['id' => $id, $label, $bundle_data];
    }

    foreach ($message as $group_id => $group) {
      $data = $group['data'];
      array_multisort(array_column($data, 'id'), SORT_ASC, $data);
      $message[$group_id]['data'] = $data;
    }

    return [
      'message' => $message,
    ];
  }

  /**
   * Get field definitions.
   *
   * @param array $params
   *   Entity type and bundle.
   *
   * @return array
   *   Fields definitions.
   */
  protected function fieldDefinitions(array $params): array {
    $entity_id = $params[0];
    $bundle = $params[1] ?? $entity_id;

    try {
      $definitions = $this->foxCommandsHelper()
        ->entityFieldManager()
        ->getFieldDefinitions($entity_id, $bundle);
    }
    catch (\Exception $e) {
      return [
        'error' => $e->getMessage(),
      ];
    }

    $header = [$this->t('ID'), $this->t('Name')];
    $data = [];
    foreach ($definitions as $field) {
      $data[] = [$field->getName(), $field->getLabel()];
    }

    $message[(string) $this->t('Fields')] = [
      'header' => $header,
      'data' => $data,
    ];

    return [
      'message' => $message,
    ];
  }

}
