<?php

namespace Drupal\fox\Plugin\FoxCommand;

/**
 * SEEK fox command.
 *
 * @FoxCommand(
 *   id = "seek",
 *   label = @Translation("Seek context record. Usage: SEEK id")
 * )
 */
class FoxCommandSeek extends FoxCommandBaseClass {

  /**
   * {@inheritdoc}
   */
  public function execute(array $params, array $variables): array {
    if (empty($params)) {
      return $this->errorReturn($this->t('Empty parameter. Usage: SEEK id'));
    }
    $id = reset($params);

    $helper = $this->foxCommandsHelper();

    $variables['id'] = $id;
    $entity = $helper->getEntity($variables);
    if (is_array($entity) and isset($entity['error'])) {
      return $entity;
    }
    if (empty($entity)) {
      return $this->errorReturn($this->t('ID @id not found', [
        '@id' => $id,
      ]));
    }

    // Get record number.
    $entity_type = $variables['entity_type'] ?? NULL;
    if (empty($entity_type)) {
      return $this->errorReturn($this->t('Empty entity type. Use USE command.'));
    }
    $bundle = $variables['bundle'] ?? NULL;
    $query = $helper->getEntityQuery($entity_type, $bundle);

    $id_key = $helper->entityKey($entity_type, 'id');
    $query->sort($id_key);
    $ids = $query->execute();
    $recno = array_search($id, array_values($ids));

    return [
      'message' => $this->t('Found'),
      'variables' => [
        'id' => $id,
        'recno' => $recno,
      ],
    ];
  }

}
