<?php

namespace Drupal\fox\Plugin\FoxCommand;

/**
 * SET fox command.
 *
 * @FoxCommand(
 *   id = "set",
 *   label = @Translation("Set variables. Usage: SET var1 WITH eExpr1 [, var2 WITH eExpr2]")
 * )
 */
class FoxCommandSet extends FoxCommandBaseClass {

  /**
   * {@inheritdoc}
   */
  public function execute(array $params, array $variables): array {
    if (empty($params)) {
      return $this->errorReturn($this->t('Empty SET variables list'));
    }

    $new_variables = [];
    $helper = $this->foxCommandsHelper();
    $items = $helper->prepareParameters($params);
    foreach ($items as $item) {
      $value = explode(' ', trim($item));
      if (count($value) < 3) {
        return $this->errorReturn($this->t('Bad SET format'));
      }

      $name = array_shift($value);
      $with = array_shift($value);

      $with = strtolower($with);
      if ($with !== 'with') {
        return $this->errorReturn($this->t('There is no WITH part'));
      }

      $value = $helper->prepareValue($value);

      if (isset($variables[$value])) {
        $value = $variables[$value];
      }

      $new_variables[] = [$name => $value];
    }

    $message = $this->t('Variables were set');

    return [
      'message' => $message,
      'variables' => $new_variables,
    ];
  }

}
